%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THIS PROGRAM COMPUTES THE THEORETICAL MOMENTS AND THE VECTOR OF DISTANCES  
% BETWEEN THEORETICAL AND EMPIRICAL MOMENTS TO BE MINIMIZED.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function f = NLS_fct(x)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP A: EXTRACT CURRENT PARAMETER VALUES AND COMPUTE AUXILIARY PARAMETERS
%         NEEDED IN COMPUTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sigma_alpha = x(1);
sigma_ab = x(2);
sigma_beta = x(3);
rho = x(4);
sigma_ARinit = x(5);
sigma_u = x(6);
sigma_eps = x(7);

g0_coeff = x(8);
g1_coeff = x(9);
g2_coeff = x(10);
g3_coeff = x(11);
g4_coeff = x(12);

p_1980 = x(13);
p_1981 = x(14);
p_1982 = x(15);
p_1983 = x(16); 
p_1984 = x(17); 
p_1985 = x(18);
p_1986 = x(19);
p_1987 = x(20);
p_1988 = x(21);
p_1989 = x(22);
p_1990 = x(23);
p_1991 = x(24);
p_1992 = x(25); 
p_1993 = x(26); 
p_1994 = x(27);
p_1995 = x(28);
p_1996 = x(29);
p_1997 = x(30);
p_1998 = x(31);
p_1999 = x(32);
p_2000 = x(33);
p_2001 = x(34); 
p_2002 = x(35); 
p_2003 = x(36);
p_2004 = x(37);

l_1980 = x(38);
l_1981 = x(39);
l_1982 = x(40);
l_1983 = x(41); 
l_1984 = x(42); 
l_1985 = x(43);
l_1986 = x(44);
l_1987 = x(45);
l_1988 = x(46);
l_1989 = x(47);
l_1990 = x(48);
l_1991 = x(49);
l_1992 = x(50); 
l_1993 = x(51); 
l_1994 = x(52);
l_1995 = x(53);
l_1996 = x(54);
l_1997 = x(55);
l_1998 = x(56);
l_1999 = x(57);
l_2000 = x(58);
l_2001 = x(59); 
l_2002 = x(60); 
l_2003 = x(61);
l_2004 = x(62);



% EXTRACT THE VARIABLES:
global covar_glob one_glob slope_het_glob exper_glob lag_glob diagyear_glob leadyear_glob gebjahr_glob diagonal_glob
covar_impute = covar_glob; 
one = one_glob;
slope_het_vars = slope_het_glob;
exper = exper_glob;
lag = lag_glob; 
diagyear = diagyear_glob;
leadyear = leadyear_glob;
gebjahr = gebjahr_glob;
diagonal = diagonal_glob;



% COLLECT FACTOR LOADINGS IN ONE MATRIX, MAKING COMPUTATION BELOW EASIER:
P = [p_1980 p_1981 p_1982 p_1983 p_1984 p_1985 p_1986 p_1987 p_1988 p_1989 p_1990 p_1991 p_1992 ...
    p_1993 p_1994 p_1995 p_1996 p_1997 p_1998 p_1999 p_2000 p_2001 p_2002 p_2003 p_2004];

L = [l_1980 l_1981 l_1982 l_1983 l_1984 l_1985 l_1986 l_1987 l_1988 l_1989 l_1990 l_1991 l_1992 ...
    l_1993 l_1994 l_1995 l_1996 l_1997 l_1998 l_1999 l_2000 l_2001 l_2002 l_2003 l_2004];
L = [1 1 L];                                        %fix factor loadings for 1978 and 1979 at 1; concatanation makes computation below easier.



% COLLECT PARAMETERS DESCRIBING HETEROSCEDASTICITY IN ONE VECTOR:
slope_het_coeff = [sigma_ab; sigma_beta];






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP B: COMPUTE THE COHORT-SPECIFIC AR(1)-PROCESSES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The AR(1)-process is given by z(ibt) = rho*z(ib,t-1) + l(t)*v(ibt)
% with var(v(ibt)) = sigma_v^2 and initial condition var(z(ib0)) =
% sigma_v0^2. Consequently, the variances follow the process
% V(ibt) = (rho^2)*V(ib,t-1) + (l(t)^2)*sigma_v^2 
% with initial condition V(ib0) = (l(b0)^2)*sigma_v0^2.
% Note that processes differ across cohorts because each cohort faces a
% different sequence of factor loadings. As a consequence, the relevant
% variance for an individual is uniquely identified by (experience,
% cohort). I thus loop over these two variables below. In particular, I fix
% a cohort, extract the initial relevant factor loading for this cohort,
% and solve the AR(1)-process forward, in each step extracting the
% associated factor loading.


% Step 1: Experience varies on the quarterly level, but factor loadings
%         vary on annual level; bring annual factor loadings on quarterly
%         frequency by assuming that each quarter within a year is
%         associated with the same loading:
helpvar1 = ones(4,1); 
L_quart = kron(L',helpvar1);


% Prepare Loop:
nr_cohorts = max(gebjahr) - min(gebjahr) + 1;                       %the number of cohorts
AR_matrix = zeros(max(exper)+1,nr_cohorts);                         %the matrix containing cohort-specific AR(1)-processes along columns



% Step 2: For each cohort, solve AR(1)-forward, using the right factor
%         loadings at each step; the outer loop (columns) is over cohorts;
%         the inner loop (rows) is over experience:


for i = 1:nr_cohorts
    
    %Translate index i into year of birth; then use the variable to compute
    %some auxiliary variables:
    birthyear = min(gebjahr) + (i-1);                               %translate index i into birth-year                 
    max_exper = (2004 - (birthyear + 23) + 1)*4 - 1;                %highest experience for the cohort indexed by i
    min_quarter = (birthyear + 23 - 1978)*4 +1;                     %the first quarter the cohort is observed in; normalize to start at 1
                                                                    % --> can use it to extract factor loading from L_quart
    
    %Initialize cohort-specific AR(1)-process:
    lambda_init = L_quart((min_quarter),1);                         %Factor Loading for the initial condition
    update = lambda_init*sigma_ARinit;
    AR_matrix(1,i) = update;
    
    for k = 2:(max_exper+1)       
               
        lambda = L_quart((min_quarter+k-1),1);                      %Factor Loadings
        exper_help = k-1;                                           %experience starts at zero
             
        % NOTE: Would like to devide the polynomials by a large number to
        % get reasonably large coefficients for gammas (like we did for the
        % polynomials in the random walk); however, the var_z is not linear
        % in these parameters ==> normalization does not work.
        var_z = g0_coeff + g1_coeff*exper_help + g2_coeff*(exper_help^2) + ...
            + g3_coeff*(exper_help^3) + g4_coeff*(exper_help^4);
        
        var_AR = (rho^2)*update + (lambda^2)*var_z;
        AR_matrix(k,i) = var_AR;
        update = var_AR;
        
    end
      
end






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP C: MATCH THE AR_matrix TO THE EMPIRICAL AUTO-COVARIANCES
%         NOTE: The match is by birthyear and cohort!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nobs = length(covar_impute);
AR_var = zeros(Nobs,1);
min_geb = min(gebjahr);

for i = 1:Nobs
    cohort_help = gebjahr(i) - min_geb + 1;
    exper_help = exper(i) + 1;
    AR_var(i,1) = (rho^lag(i))*AR_matrix(exper_help,cohort_help);        
end






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP D: COMPUTE THE FACTOR LOADINGS ON THE PERMANENT COMPONENT; THEN
%         COMPUTE THE PERMANENT COMPONENT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Use Michael Baker's trick introducing the year-dummies for diagonal and
% lead elements as the exponent of the factor loadings to "switch on and 
% off" the factor loadings:

T = 2004-1980+1;
loadings_permanent = one;

for t = 1:T
    diag_help = P(1,t).^diagyear(:,t);
    lead_help = P(1,t).^leadyear(:,t);
    loadings_help = diag_help.*lead_help;
    loadings_permanent = loadings_permanent.*loadings_help;
end


clear diag_help lead_help loadings_help;



% Compute the permanent part:
perm_component = sigma_alpha*one + slope_het_vars*slope_het_coeff + sigma_u*(exper/10);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP E: COMPUTE THE THEORETICAL MOMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

covar_hat = loadings_permanent.*perm_component + AR_var + sigma_eps*diagonal;
clear loadings_permanent;
      




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP F: COMPUTE THE SQUARED DEVIATION BETWEEN THEORETICAL AND EMPIRICAL
%         MOMENTS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f = ((covar_impute - covar_hat)')*(covar_impute - covar_hat);









