delete('full_unithomosc_results.txt');

diary('full_unithomosc_results.txt');


tic;

clear;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THIS PROGRAM IS THE "SCRIPT" FOR THE PROGRAM THAT ESTIMATES PARAMETERS OF
% THE FULL NON-STATIONARY MODEL WITH A HOMOSCEDASTIC UNIT ROOTS PROCESS. 
% THIS VERSION USES A NON-ZERO INITIAL CONDITION FOR THE AR(1)-PROCESS. 
% I USE A CONSTRAINED MINIMIZER HERE. AFTER EXPERIMENTING WITH DIFFERENT 
% SOLVERS, INCLUDING KNITRO, I FOUND THAT FMINCON WITH SQP WORKS VERY WELL.

% MODEL SPECIFICATION: 
% (i)    RANDOM EFFECT WITH VARIANCE sigma_alpha
% (ii)   RANDOM WALK WITH HOMOSCEDASTIC SHOCKS.
% (iii)  SLOPE-RATE HETEROGENEITY
% (iv)   AR(1) WITH HETEROSCEDASTIC SHOCKS, NON-ZERO INITIAL CONDITION, AND 
%        PERSISTENCE rho.
% (v)    MEASUREMENT ERROR
% (vi)   FACTOR LOADINGS ON PERMANENT COMPONENT, p(t), AND ON AR(1), l(t).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 1: LOAD DATA AND EXTRACT VARIABLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load 'PATHNAME\autocovar_hsdeg_MATLAB.mat'
D = data;
clear data;


% THE FOLLOWING IS A LIST OF ALL VARIABLES IN THE DATA MATRIX, INCLUDING
% THOSE THAT ARE UNUSED (make sure to read the "readme" file):
%panel_var = D(:,1);
%year = D(:,2); 
%quarter = D(:,3); 
gebjahr = D(:,4); 
%covar = D(:,5); 
covar_impute = D(:,6); 
lag = D(:,7); 
%birth_1955 = D(:,8); 
%birth_1956 = D(:,9); 
%birth_1957 = D(:,10); 
%birth_1958 = D(:,11); 
%birth_1959 = D(:,12); 
%birth_1960 = D(:,13); 
%birth_1961 = D(:,14); 
%birth_1962 = D(:,15); 
%birth_1963 = D(:,16); 
%birth_1964 = D(:,17); 
%birth_1965 = D(:,18); 
%birth_1966 = D(:,19); 
%birth_1967 = D(:,20); 
%birth_1968 = D(:,21); 
%birth_1969 = D(:,22); 
%birth_1970 = D(:,23); 
%birth_1971 = D(:,24); 
%birth_1972 = D(:,25); 
%birth_1973 = D(:,26); 
%birth_1974 = D(:,27); 
%birth_1975 = D(:,28); 
%birth_1976 = D(:,29); 
%birth_1977 = D(:,30); 
%birth_1978 = D(:,31); 
%ones = D(:,32); 
diagonal = D(:,33); 
%min_time = D(:,34); 
exper = D(:,35); 
%exper_square = D(:,36); 
%exper_double = D(:,37); 
%d1_var = D(:,38); 
%d2_var = D(:,39); 
%d3_var = D(:,40); 
%d4_var = D(:,41); 
sigma_ab_var = D(:,42); 
sigma_b_var = D(:,43); 
%lead_year = D(:,44); 
diagyear_1980 = D(:,45); 
leadyear_1980 = D(:,46); 
diagyear_1981 = D(:,47); 
leadyear_1981 = D(:,48); 
diagyear_1982 = D(:,49); 
leadyear_1982 = D(:,50); 
diagyear_1983 = D(:,51); 
leadyear_1983 = D(:,52); 
diagyear_1984 = D(:,53); 
leadyear_1984 = D(:,54); 
diagyear_1985 = D(:,55); 
leadyear_1985 = D(:,56); 
diagyear_1986 = D(:,57); 
leadyear_1986 = D(:,58); 
diagyear_1987 = D(:,59); 
leadyear_1987 = D(:,60); 
diagyear_1988 = D(:,61); 
leadyear_1988 = D(:,62); 
diagyear_1989 = D(:,63); 
leadyear_1989 = D(:,64); 
diagyear_1990 = D(:,65); 
leadyear_1990 = D(:,66); 
diagyear_1991 = D(:,67); 
leadyear_1991 = D(:,68); 
diagyear_1992 = D(:,69); 
leadyear_1992 = D(:,70); 
diagyear_1993 = D(:,71); 
leadyear_1993 = D(:,72); 
diagyear_1994 = D(:,73); 
leadyear_1994 = D(:,74); 
diagyear_1995 = D(:,75); 
leadyear_1995 = D(:,76); 
diagyear_1996 = D(:,77); 
leadyear_1996 = D(:,78); 
diagyear_1997 = D(:,79); 
leadyear_1997 = D(:,80); 
diagyear_1998 = D(:,81); 
leadyear_1998 = D(:,82); 
diagyear_1999 = D(:,83); 
leadyear_1999 = D(:,84); 
diagyear_2000 = D(:,85); 
leadyear_2000 = D(:,86); 
diagyear_2001 = D(:,87); 
leadyear_2001 = D(:,88); 
diagyear_2002 = D(:,89); 
leadyear_2002 = D(:,90); 
diagyear_2003 = D(:,91); 
leadyear_2003 = D(:,92); 
diagyear_2004 = D(:,93); 
leadyear_2004 = D(:,94);

clear D;

Nobs = length(covar_impute);                %Total number of observations
one = ones(Nobs,1); 



leadyear = [leadyear_1980 leadyear_1981 leadyear_1982 leadyear_1983 leadyear_1984 leadyear_1985 leadyear_1986 leadyear_1987 ...
    leadyear_1988 leadyear_1989 leadyear_1990 leadyear_1991 leadyear_1992 leadyear_1993 leadyear_1994 leadyear_1995 leadyear_1996 ...
    leadyear_1997 leadyear_1998 leadyear_1999 leadyear_2000 leadyear_2001 leadyear_2002 leadyear_2003 leadyear_2004];
clear leadyear_1980 leadyear_1981 leadyear_1982 leadyear_1983 leadyear_1984 leadyear_1985 leadyear_1986 leadyear_1987 ...
    leadyear_1988 leadyear_1989 leadyear_1990 leadyear_1991 leadyear_1992 leadyear_1993 leadyear_1994 leadyear_1995 leadyear_1996 ...
    leadyear_1997 leadyear_1998 leadyear_1999 leadyear_2000 leadyear_2001  leadyear_2002  leadyear_2003  leadyear_2004;

diagyear = [diagyear_1980 diagyear_1981 diagyear_1982 diagyear_1983 diagyear_1984 diagyear_1985 diagyear_1986 diagyear_1987 ...
    diagyear_1988 diagyear_1989 diagyear_1990 diagyear_1991 diagyear_1992 diagyear_1993 diagyear_1994 diagyear_1995 diagyear_1996 ...
    diagyear_1997 diagyear_1998 diagyear_1999 diagyear_2000 diagyear_2001 diagyear_2002 diagyear_2003 diagyear_2004];
clear diagyear_1980 diagyear_1981 diagyear_1982 diagyear_1983 diagyear_1984 diagyear_1985 diagyear_1986 diagyear_1987 ...
    diagyear_1988 diagyear_1989 diagyear_1990 diagyear_1991 diagyear_1992 diagyear_1993 diagyear_1994 diagyear_1995 diagyear_1996 ...
    diagyear_1997 diagyear_1998 diagyear_1999 diagyear_2000 diagyear_2001  diagyear_2002  diagyear_2003  diagyear_2004;




% Save the variables multiplying parameters that describe
% slope-heterogeneity and heteroscedasticity in matrices and scale them to
% avoid over-or underflows;
% NOTE: For the permanent component, this procedure works since it is
% linear in parameters. The same is NOT true for the AR(1)-component.
slope_het_vars = [sigma_ab_var/10 sigma_b_var/1000];




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 2: INITIALIZE PARAMETERS
%         NOTE: I experimented a lot with starting values and different 
%         solvers. KNITRO turned out to be sensitive to starting values. 
%         As it turns out, fmincon with sqp works well. The main source of
%         numerical instability is the measurement error component. I thus
%         use the parameter estimates from a model without measurement
%         error as starting values. These starting values are contained in
%         the sub-folder.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Starting values:
load 'PATHNAME\estimates_fullunitrootshomosc.mat'

x_noslope = x;


% VARIANCE PARAMETERS:
sigma_alpha = x_noslope(1);
sigma_ab = 0;
sigma_beta = 0;
rho = x_noslope(4);
sigma_ARinit = x_noslope(5);
sigma_u = x_noslope(6);
sigma_eps = 0;



% PARAMETERS FOR HETEROSCEDASTIC AR(1):
g0_coeff = x_noslope(7);
g1_coeff = x_noslope(8);
g2_coeff = x_noslope(9);
g3_coeff = x_noslope(10);
g4_coeff = x_noslope(11);


% FACTOR LOADINGS FOR PERMANENT COMPONENT:
p_1980 = x_noslope(12);
p_1981 = x_noslope(13);
p_1982 = x_noslope(14);
p_1983 = x_noslope(15); 
p_1984 = x_noslope(16); 
p_1985 = x_noslope(17);
p_1986 = x_noslope(18);
p_1987 = x_noslope(19);
p_1988 = x_noslope(20);
p_1989 = x_noslope(21);
p_1990 = x_noslope(22);
p_1991 = x_noslope(23);
p_1992 = x_noslope(24); 
p_1993 = x_noslope(25); 
p_1994 = x_noslope(26);
p_1995 = x_noslope(27);
p_1996 = x_noslope(28);
p_1997 = x_noslope(29);
p_1998 = x_noslope(30);
p_1999 = x_noslope(31);
p_2000 = x_noslope(32);
p_2001 = x_noslope(33); 
p_2002 = x_noslope(34); 
p_2003 = x_noslope(35);
p_2004 = x_noslope(36);




% FACTOR LOADINGS FOR PERSISTENT COMPONENT:
l_1980 = x_noslope(37);
l_1981 = x_noslope(38);
l_1982 = x_noslope(39);
l_1983 = x_noslope(40); 
l_1984 = x_noslope(41); 
l_1985 = x_noslope(42);
l_1986 = x_noslope(43);
l_1987 = x_noslope(44);
l_1988 = x_noslope(45);
l_1989 = x_noslope(46);
l_1990 = x_noslope(47);
l_1991 = x_noslope(48);
l_1992 = x_noslope(49); 
l_1993 = x_noslope(50); 
l_1994 = x_noslope(51);
l_1995 = x_noslope(52);
l_1996 = x_noslope(53);
l_1997 = x_noslope(54);
l_1998 = x_noslope(55);
l_1999 = x_noslope(56);
l_2000 = x_noslope(57);
l_2001 = x_noslope(58); 
l_2002 = x_noslope(59); 
l_2003 = x_noslope(60);
l_2004 = x_noslope(61);


X0 = [sigma_alpha sigma_ab sigma_beta rho sigma_ARinit sigma_u sigma_eps ...
    g0_coeff g1_coeff g2_coeff g3_coeff g4_coeff ...
    p_1980 p_1981 p_1982 p_1983 p_1984 p_1985 p_1986 p_1987 p_1988 p_1989 p_1990 p_1991 ...
    p_1992 p_1993 p_1994 p_1995 p_1996 p_1997 p_1998 p_1999 p_2000 p_2001 p_2002 p_2003 p_2004 ...
    l_1980 l_1981 l_1982 l_1983 l_1984 l_1985 l_1986 l_1987 l_1988 l_1989 l_1990 l_1991 ...
    l_1992 l_1993 l_1994 l_1995 l_1996 l_1997 l_1998 l_1999 l_2000 l_2001 l_2002 l_2003 l_2004]';





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 3: COMPUTE THE CONSTRAINT
%         NOTE: There are two sets of constraints other than 
%               non-negativity of variance parameters/factor loadings:
%               var(alpha+beta*exper)>=0
%               var(z)>=0
%               This can be written as A*X > =0. However, the routine
%               requires one to formulate it in terms of "<=". 
%               There are 62 parameters, [max(exper)+ 1] restrictions 
%               on the variance of the permanent component and max(exper)
%               restrictions on the persistent component
%               (var(u) = var(z) = 0 at exper=0!). The var(alpha)
%               is already restriced by the restriction on 
%               var(alpha+beta*exper) evaluated at exper=0,
%               and covar(alpha, beta) can be positive or negative and 
%               remains unrestricted. However, we do need to directly 
%               impose non-negativity on var(beta). Furthermore, imposing
%               var(z)>=0 implies that no further restrictions need 
%               to be imposed on the g-parameters. Hence, there are  
%               55 additional  non-negativity constraints, and thus:
%               N_con = [2*max(exper)+ 1] + 55
%               restrictions, and A is of size (N_con)x(N_param).    
%         NOTE2: It is important to force the constraints to be satisfied
%               out of sample as well. I set max_exper = 160, which allows
%               for a life-cycle of 40 years/160 quarters.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Prepare the matrix A:
max_exper = 160;
N_param = length(X0);
N_con = 2*max_exper + 1 + 55;
A = zeros(N_con,N_param);


% Generate the variables involving the exper:
exper_vec = [0:max_exper]';
ones_vec = ones(length(exper_vec),1);
slope_vec = [ones_vec (2*exper_vec)/10 (exper_vec.^2)/1000];

exper_vec = [1:max_exper]';                                 %note that we do not need to evaluated var(u) and var(z) at exper = 0!
ones_vec = ones(length(exper_vec),1);
pers_vec = [ones_vec exper_vec exper_vec.^2 exper_vec.^3 exper_vec.^4];


% Now fill in the elements of A, following the order of parameters in X0:
A(1,3) = 1;                                                                             %non-negativity of var(beta)
A(2:(length(slope_vec)+1),1:3) = slope_vec;                                             %non-negativity of permanent component
A(length(slope_vec)+2,4) = 1;                                                           %non-negativity of rho
A(length(slope_vec)+3,5) = 1;                                                           %non-negativity of initial condition
A(length(slope_vec)+4,6) = 1;                                                           %non-negativity of sigma_u
A(length(slope_vec)+5,7) = 1;                                                           %non-negativity of sigma_eps
A((length(slope_vec)+6):(length(slope_vec)+6+length(pers_vec)-1),8:12) = pers_vec;      %non-negativity of var(z)
A((length(A)-49):length(A),13:62) = eye(50);                                            %non-negativity of factor loadings


% Translate into non-positivity constraint and compute RHS of constraint:
A = -A;
b = zeros(length(A),1);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 4: MINIMIZE THE DISTANCE BETWEEN EMPIRICAL AND THEORETICAL MOMENTS.
%         NOTE: The Knitro-algorithms do not work well, diverging from the
%               starting point to some solution with a much larger function
%               value. I have experimented with many different starting
%               values, but non converge to a solution that is anywhere 
%               close to the unconstrained estimates. I thus have also 
%               experimented with some of the algorithms coming with the 
%               fmincon-command. As it turns out, the sqp-algorithm works
%               extremely well here, first jumping a bit in value, then 
%               slowly converging to a function value very close to the
%               one found with the unconstrained  optimization. This works 
%               well for different starting values, though convergence is 
%               generally slow. I stop the algorithm below after 800
%               iterations since afterwards it does not improve at all
%               anymore.
%               In case a user has extended licenses (e.g. KNITRO), the
%               program needs to be modified here. For knitro, this is
%               straightforward, since the command "knitromatlab" has the
%               same syntax like fmincon.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% THE VARIABLES ARE PASSED ON AS GLOBALS. IN CASES USERS WANT TO EXPAND THE
% MODEL TO INCLUDE MORE VARIABLES, IT MAY BE WORTH CONSIDERING PASSING THEM
% ON THROUGH A FUNCTION HANDLE.
global covar_glob one_glob slope_het_glob exper_glob lag_glob diagyear_glob leadyear_glob gebjahr_glob diagonal_glob
covar_glob = covar_impute;
one_glob = one;
slope_het_glob = slope_het_vars;
exper_glob = exper;
lag_glob = lag;
diagyear_glob = diagyear;
leadyear_glob = leadyear;
gebjahr_glob = gebjahr;
diagonal_glob = diagonal;


% SET OPTIONS. MAKE SURE TO USE TWO-SIDED FINITE DIFFERENCES!
options_help = optimset('fmincon');
options = optimset(options_help,'Algorithm','sqp','display','iter','MaxIter',1000,'TolCon',1.0000e-010, ...
    'FinDiffType','central','TolX',1.0000e-030,'MaxFunEvals',300000);
x = fmincon(@NLS_fct,X0,A,b,[],[],[],[],[],options);



% SAVE THE ESTIMATES IN A MATRIX:
save('VARNAME','x');

toc;
t = toc;
fprintf('%10d\n', t);


diary off;

