%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THIS PROGRAM EVALUATES THE THEORETICAL AUTOCOVARIANCE STRUCTURE AT SOME
% PARAMETER VECTOR. THE RESULT IS A VECTOR OF LENGTH N CONTAINING THE 
% PREDICTED AUTOCOVARIANCES. THIS CAN BE USED TO CONSTRUCT THE RESIDUALS, 
% OR TO COMPUTE THE NUMERICAL GRADIENT.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function covar_help = evaluate_covar(paramet,one,slope_het_vars,exper,lag,diagyear,leadyear,gebjahr,diagonal)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP A: EXTRACT CURRENT PARAMETER VALUES AND COMPUTE AUXILIARY PARAMETERS
%         NEEDED IN COMPUTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sigma_alpha = paramet(1);
sigma_ab = paramet(2);
sigma_beta = paramet(3);
rho = paramet(4);
sigma_ARinit = paramet(5);
sigma_u = paramet(6);
sigma_eps = paramet(7);

g0_coeff = paramet(8);
g1_coeff = paramet(9);
g2_coeff = paramet(10);
g3_coeff = paramet(11);
g4_coeff = paramet(12);

p_1980 = paramet(13);
p_1981 = paramet(14);
p_1982 = paramet(15);
p_1983 = paramet(16); 
p_1984 = paramet(17); 
p_1985 = paramet(18);
p_1986 = paramet(19);
p_1987 = paramet(20);
p_1988 = paramet(21);
p_1989 = paramet(22);
p_1990 = paramet(23);
p_1991 = paramet(24);
p_1992 = paramet(25); 
p_1993 = paramet(26); 
p_1994 = paramet(27);
p_1995 = paramet(28);
p_1996 = paramet(29);
p_1997 = paramet(30);
p_1998 = paramet(31);
p_1999 = paramet(32);
p_2000 = paramet(33);
p_2001 = paramet(34); 
p_2002 = paramet(35); 
p_2003 = paramet(36);
p_2004 = paramet(37);

l_1980 = paramet(38);
l_1981 = paramet(39);
l_1982 = paramet(40);
l_1983 = paramet(41); 
l_1984 = paramet(42); 
l_1985 = paramet(43);
l_1986 = paramet(44);
l_1987 = paramet(45);
l_1988 = paramet(46);
l_1989 = paramet(47);
l_1990 = paramet(48);
l_1991 = paramet(49);
l_1992 = paramet(50); 
l_1993 = paramet(51); 
l_1994 = paramet(52);
l_1995 = paramet(53);
l_1996 = paramet(54);
l_1997 = paramet(55);
l_1998 = paramet(56);
l_1999 = paramet(57);
l_2000 = paramet(58);
l_2001 = paramet(59); 
l_2002 = paramet(60); 
l_2003 = paramet(61);
l_2004 = paramet(62);



% COLLECT FACTOR LOADINGS IN ONE MATRIX, MAKING COMPUTATION BELOW EASIER:
P = [p_1980 p_1981 p_1982 p_1983 p_1984 p_1985 p_1986 p_1987 p_1988 p_1989 p_1990 p_1991 p_1992 ...
    p_1993 p_1994 p_1995 p_1996 p_1997 p_1998 p_1999 p_2000 p_2001 p_2002 p_2003 p_2004];

L = [l_1980 l_1981 l_1982 l_1983 l_1984 l_1985 l_1986 l_1987 l_1988 l_1989 l_1990 l_1991 l_1992 ...
    l_1993 l_1994 l_1995 l_1996 l_1997 l_1998 l_1999 l_2000 l_2001 l_2002 l_2003 l_2004];
L = [1 1 L];                                        %fix factor loadings for 1978 and 1979 at 1; concatanation makes computation below easier.



% COLLECT PARAMETERS DESCRIBING HETEROSCEDASTICITY IN ONE VECTOR:
slope_het_coeff = [sigma_ab; sigma_beta];






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP B: COMPUTE THE COHORT-SPECIFIC AR(1)-PROCESSES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The AR(1)-process is given by z(ibt) = rho*z(ib,t-1) + l(t)*v(ibt)
% with var(v(ibt)) = sigma_v^2 and initial condition var(z(ib0)) =
% sigma_v0^2. Consequently, the variances follow the process
% V(ibt) = (rho^2)*V(ib,t-1) + (l(t)^2)*sigma_v^2 
% with initial condition V(ib0) = (l(b0)^2)*sigma_v0^2.
% Note that processes differ across cohorts because each cohort faces a
% different sequence of factor loadings. As a consequence, the relevant
% variance for an individual is uniquely identified by (experience,
% cohort). I thus loop over these two variables below. In particular, I fix
% a cohort, extract the initial relevant factor loading for this cohort,
% and solve the AR(1)-process forward, in each step extracting the
% associated factor loading.


% Step 1: Experience varies on the quarterly level, but factor loadings
%         vary on annual level; bring annual factor loadings on quarterly
%         frequency by assuming that each quarter within a year is
%         associated with the same loading:
helpvar1 = ones(4,1); 
L_quart = kron(L',helpvar1);


% Prepare Loop:
nr_cohorts = max(gebjahr) - min(gebjahr) + 1;                       %the number of cohorts
AR_matrix = zeros(max(exper)+1,nr_cohorts);                         %the matrix containing cohort-specific AR(1)-processes along columns



% Step 2: For each cohort, solve AR(1)-forward, using the right factor
%         loadings at each step; the outer loop (columns) is over cohorts;
%         the inner loop (rows) is over experience:


for i = 1:nr_cohorts
    
    %Translate index i into year of birth; then use the variable to compute
    %some auxiliary variables:
    birthyear = min(gebjahr) + (i-1);                               %translate index i into birth-year                 
    max_exper = (2004 - (birthyear + 23) + 1)*4 - 1;                %highest experience for the cohort indexed by i
    min_quarter = (birthyear + 23 - 1978)*4 +1;                     %the first quarter the cohort is observed in; normalize to start at 1
                                                                    % --> can use it to extract factor loading from L_quart
    
    %Initialize cohort-specific AR(1)-process:
    lambda_init = L_quart((min_quarter),1);                         %Factor Loading for the initial condition
    update = lambda_init*sigma_ARinit;
    AR_matrix(1,i) = update;
    
    for k = 2:(max_exper+1)       
               
        lambda = L_quart((min_quarter+k-1),1);                      %Factor Loadings
        exper_help = k-1;                                           %experience starts at zero
             
        % NOTE: Would like to devide the polynomials by a large number to
        % get reasonably large coefficients for gammas (like we did for the
        % polynomials in the random walk); however, the var_z is not linear
        % in these parameters ==> normalization does not work.
        var_z = g0_coeff + g1_coeff*exper_help + g2_coeff*(exper_help^2) + ...
            + g3_coeff*(exper_help^3) + g4_coeff*(exper_help^4);
        
        var_AR = (rho^2)*update + (lambda^2)*var_z;
        AR_matrix(k,i) = var_AR;
        update = var_AR;
        
    end
      
end






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP C: MATCH THE AR_matrix TO THE EMPIRICAL AUTO-COVARIANCES
%         NOTE: The match is by birthyear and cohort!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nobs = length(exper);
AR_var = zeros(Nobs,1);
min_geb = min(gebjahr);

for i = 1:Nobs
    cohort_help = gebjahr(i) - min_geb + 1;
    exper_help = exper(i) + 1;
    AR_var(i,1) = (rho^lag(i))*AR_matrix(exper_help,cohort_help);        
end






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP D: COMPUTE THE FACTOR LOADINGS ON THE PERMANENT COMPONENT; THEN
%         COMPUTE THE PERMANENT COMPONENT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Use Michael Baker's trick introducing the year-dummies for diagonal and
% lead elements as the exponent of the factor loadings to "switch on and 
% off" the factor loadings:

T = 2004-1980+1;
loadings_permanent = one;

for t = 1:T
    diag_help = P(1,t).^diagyear(:,t);
    lead_help = P(1,t).^leadyear(:,t);
    loadings_help = diag_help.*lead_help;
    loadings_permanent = loadings_permanent.*loadings_help;
end


clear diag_help lead_help loadings_help;



% Compute the permanent part:
perm_component = sigma_alpha*one + slope_het_vars*slope_het_coeff + sigma_u*(exper/10);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP E: COMPUTE THE THEORETICAL MOMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

covar_hat = loadings_permanent.*perm_component + AR_var + sigma_eps*diagonal;
covar_help = covar_hat;









