delete('fullunithomosc_table.txt');

diary('fullunithomosc_table.txt');

clear;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THIS PROGRAM COMPUTES A TABLE WITH THE PARAMETER ESTIMATES, THEIR
% STANDARD ERRORS, THE R^2 AND A JOINT WALD TEST FOR SLOPE HETEROGENEITY. 
% STANDARD ERRORS WILL BE THOSE FOR NLS, BUT WITH CLUSTERING WITHIN COHORT.
% NOTE THAT THE DATA ARE ALREADY SORTED BY COHORT!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 1: LOAD DATA AND PARAMETER ESTIMATES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load the data matrix:
load 'PATHNAME\autocovar_hsdeg_MATLAB.mat'
D = data;
clear data;


gebjahr = D(:,4); 
covar_impute = D(:,6); 
lag = D(:,7); 

diagonal = D(:,33); 
exper = D(:,35); 
sigma_ab_var = D(:,42); 
sigma_b_var = D(:,43); 
diagyear_1980 = D(:,45); 
leadyear_1980 = D(:,46); 
diagyear_1981 = D(:,47); 
leadyear_1981 = D(:,48); 
diagyear_1982 = D(:,49); 
leadyear_1982 = D(:,50); 
diagyear_1983 = D(:,51); 
leadyear_1983 = D(:,52); 
diagyear_1984 = D(:,53); 
leadyear_1984 = D(:,54); 
diagyear_1985 = D(:,55); 
leadyear_1985 = D(:,56); 
diagyear_1986 = D(:,57); 
leadyear_1986 = D(:,58); 
diagyear_1987 = D(:,59); 
leadyear_1987 = D(:,60); 
diagyear_1988 = D(:,61); 
leadyear_1988 = D(:,62); 
diagyear_1989 = D(:,63); 
leadyear_1989 = D(:,64); 
diagyear_1990 = D(:,65); 
leadyear_1990 = D(:,66); 
diagyear_1991 = D(:,67); 
leadyear_1991 = D(:,68); 
diagyear_1992 = D(:,69); 
leadyear_1992 = D(:,70); 
diagyear_1993 = D(:,71); 
leadyear_1993 = D(:,72); 
diagyear_1994 = D(:,73); 
leadyear_1994 = D(:,74); 
diagyear_1995 = D(:,75); 
leadyear_1995 = D(:,76); 
diagyear_1996 = D(:,77); 
leadyear_1996 = D(:,78); 
diagyear_1997 = D(:,79); 
leadyear_1997 = D(:,80); 
diagyear_1998 = D(:,81); 
leadyear_1998 = D(:,82); 
diagyear_1999 = D(:,83); 
leadyear_1999 = D(:,84); 
diagyear_2000 = D(:,85); 
leadyear_2000 = D(:,86); 
diagyear_2001 = D(:,87); 
leadyear_2001 = D(:,88); 
diagyear_2002 = D(:,89); 
leadyear_2002 = D(:,90); 
diagyear_2003 = D(:,91); 
leadyear_2003 = D(:,92); 
diagyear_2004 = D(:,93); 
leadyear_2004 = D(:,94);

clear D;

Nobs = length(covar_impute);                %Total number of observations
one = ones(Nobs,1); 

leadyear = [leadyear_1980 leadyear_1981 leadyear_1982 leadyear_1983 leadyear_1984 leadyear_1985 leadyear_1986 leadyear_1987 ...
    leadyear_1988 leadyear_1989 leadyear_1990 leadyear_1991 leadyear_1992 leadyear_1993 leadyear_1994 leadyear_1995 leadyear_1996 ...
    leadyear_1997 leadyear_1998 leadyear_1999 leadyear_2000 leadyear_2001 leadyear_2002 leadyear_2003 leadyear_2004];
clear leadyear_1980 leadyear_1981 leadyear_1982 leadyear_1983 leadyear_1984 leadyear_1985 leadyear_1986 leadyear_1987 ...
    leadyear_1988 leadyear_1989 leadyear_1990 leadyear_1991 leadyear_1992 leadyear_1993 leadyear_1994 leadyear_1995 leadyear_1996 ...
    leadyear_1997 leadyear_1998 leadyear_1999 leadyear_2000 leadyear_2001  leadyear_2002  leadyear_2003  leadyear_2004;

diagyear = [diagyear_1980 diagyear_1981 diagyear_1982 diagyear_1983 diagyear_1984 diagyear_1985 diagyear_1986 diagyear_1987 ...
    diagyear_1988 diagyear_1989 diagyear_1990 diagyear_1991 diagyear_1992 diagyear_1993 diagyear_1994 diagyear_1995 diagyear_1996 ...
    diagyear_1997 diagyear_1998 diagyear_1999 diagyear_2000 diagyear_2001 diagyear_2002 diagyear_2003 diagyear_2004];
clear diagyear_1980 diagyear_1981 diagyear_1982 diagyear_1983 diagyear_1984 diagyear_1985 diagyear_1986 diagyear_1987 ...
    diagyear_1988 diagyear_1989 diagyear_1990 diagyear_1991 diagyear_1992 diagyear_1993 diagyear_1994 diagyear_1995 diagyear_1996 ...
    diagyear_1997 diagyear_1998 diagyear_1999 diagyear_2000 diagyear_2001  diagyear_2002  diagyear_2003  diagyear_2004;



slope_het_vars = [sigma_ab_var/10 sigma_b_var/1000];



% Load the vector "x" of parameter estimates; needs to be a column vector!
load 'PATHNAME\VARNAME.mat'





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 2: EVALUATE THE COVARIANCES AT THE FOLLOWING PARAMETER VECTORS:
%       (i)  The estimates (for the residuals and the R^2)
%       (ii) The upward and downward perturbations (for the scores).
%         As a consequence, there will be vectors of predicted covariances
%         of length N for each parameter combinations. Point (i) gives
%         exactly one such vector, but (ii) will yield 2*K perturbations
%         where K is the number of parameters. Hence, the function below 
%         will return a matrix of size Nx(2K+1).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First set the size of the perturbation; look at Judd (p. 280): the upper
% bound depends on the accuracy of the computed value of f(x), which is by
% default e(-6). It also depends on the upper bound of the second 
% derivative, which I do not know. Since the partial second derivatives are 
% different for each parameter, the step length can vary by parameter. Here
% I just perturb by a fraction of the parameter. 
% Write the step size into a matrix such that it can be extracted in the
% loop below. The first length(x) elements correspond to the upward
% perturbation, the second length(x) elements to the downward perturbation.
% Note that in this way I can let the stepsize vary by parameter!
perturb_fraction = 1/10000;
perturb_help = perturb_fraction*abs(x);
perturb_vec = [perturb_help; -perturb_help];




% EVALUATE THE VECTORIZED THEORETICAL COVARIANCE STRUCTURE (i.e. THE
% PREDICTION) AT THE ESTIMATED PARAMETERS AND THE PERTURBATIONS THAT ARE
% REQUIRED FOR THE NUMERICAL GRADIENT:
nK = (2*length(x)) + 1;
covar_eval = zeros(length(covar_impute),nK);

for k = 1:nK
    
    % The if-loop determines which element in the parameter vector will be
    % perturbed and if it is a positive or negative perturbation.
    
    if k == nK
        paramet = x;                                    % this is the case where I evaluate the covariance at the estimated parameter
    else
        l = floor((k-1)/length(x));                     % here I construct a running index that determines which parameter will be perturbed
        l = k - l*length(x);
        paramet = x;
        paramet(l) = x(l) + perturb_vec(k);
    end
    
    covar_help = evaluate_covar(paramet,one,slope_het_vars,exper,lag,diagyear,leadyear,gebjahr,diagonal);
    covar_eval(:,k) = covar_help;
    clear covar_help;

end







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 3: COMPUTE THE NUMERICAL GRADIENT AT EACH OBSERVATION (THE "SCORE")
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This is a matrix of size Nobs x K, where K is the number of structural
% parameters. See e.g. Cameron&Trivedi pp. 154. Note that the covariances
% computed above are first evaluated at the K positive perturbations, 
% then at the K negative perturbations.

nK = length(x);
gradient_covar = zeros(length(covar_impute),nK);

for k = 1:nK
    covar_U = covar_eval(:,k);                  %covar_hat if the kth parameter is perturbed to theta(k)+D
    covar_L = covar_eval(:,k+nK);               %covar_hat if the kth parameter is perturbed to theta(k)-D
    
    perturb_help = abs(perturb_vec(k));
    
    gradient_hat = (covar_U - covar_L)/(2*perturb_help);
    gradient_covar(:,k) = gradient_hat;
    clear gradient_hat
end





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 4: COMPUTE THE STANDARD ERRORS, CLUSTERED BY COHORT
%         % NOTE: Since this specification does not hit the constraints, I
%                 can compare the my clustered standard errors with those 
%                 computed by Stata. They are almost identical.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First compute the residuals:
covar_hat = covar_eval(:,length(covar_eval(1,:)));
u_hat = covar_impute - covar_hat;


% NOTE: DATA ARE ALREADY SORTED BY COHORT!
n1 = min(gebjahr);
n2 = max(gebjahr);

sandwich_outer = zeros(length(x),length(x)); 
sandwich_inner = zeros(length(x),length(x));


for c = n1:n2
    cl = find(gebjahr == c,1);
    cu = find(gebjahr == c,1,'last');
    
    grad_c = gradient_covar(cl:cu,:);
    res_c = u_hat(cl:cu,:);
    
    score_c = (grad_c')*grad_c;
    score_res1 = (grad_c')*res_c;
    score_res2 = (res_c')*grad_c;
    score_res = score_res1*score_res2;
    
    sandwich_outer = sandwich_outer + score_c;
    sandwich_inner = sandwich_inner + score_res;  
end

var_cov = (inv(sandwich_outer))*sandwich_inner*(inv(sandwich_outer));
std_err = sqrt(diag(var_cov));
t_stat = x./std_err;






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 5: DISPLAY RESULTS, INCLUDING R-SQUARED AND A WALD-TES FOR SLOPE
%         HETEROGENEITY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% FIRST COMPUTE THE R^2:
TSS = ((covar_impute - mean(covar_impute))')*(covar_impute - mean(covar_impute));
RSS = (u_hat')*u_hat;
R_2 = 1 - (RSS/TSS);



% NEXT, CARRY OUT THE WALD TEST FOR EXCLUSION RESTRICTION (see e.g.
% Cameron&Trivedi, p. 228):
theta_hat = x(2:3);
var_theta = var_cov(2:3,2:3);
W = theta_hat'*(inv(var_theta))*theta_hat;
P_value = 1 - chi2cdf(W,length(theta_hat));





% DISPLAY ESTIMATION RESULTS -  ESTIMATE, STANDARD ERROR AND T-STAT:

helpnumber = length(x);

disp(' ');
disp('*******************************************************************');
disp('*******************************************************************');
disp('RESULTS FROM THE FULL MODEL WITH A HOMOSCEDASTIC UNIT ROOTS: ');
disp('PARAMETER ESTIMATES, STANDARD ERROR, AND T-STAT');
disp('********************************************************************');
disp('********************************************************************');
disp(' ');
disp('-------------------------------------');
disp('    Est         Std       t-stat');

for r=1:helpnumber;
    fprintf('%10.4f %10.4f %10.4f\n', [x(r,1) std_err(r,1) t_stat(r,1)]);
end
disp('-------------------------------------');
disp(' ');
disp('THE R-SQUARED IS:');
fprintf('%10d\n', R_2);
disp(' ');
disp('THE P-VALUE FOR A TEST OF SLOPE-HETEROGENEITY IS:');
fprintf('%10d\n', P_value);
disp(' ');
disp('THE NUMBER OF OBSERVATIONS USED IN THE ESTIMATION ROUTINE IS:');
fprintf('%10d\n', Nobs(1,1));
disp('*******************************************************************');
disp('*******************************************************************');


diary off;
